import sys
import re


class ConfigurationManager:
    @staticmethod
    def is_valid_format(s):
        # Define the regex pattern for individual pairs
        pair_pattern = r'^(sw|udh|ub|r|pc|u|ui|uvcs)[_]\d+(-\d+)?,(sw|udh|ub|r|pc|u|ui|uvcs)[_]\d+(-\d+)?$'

        # Split the string by ';'
        pairs = s.split(';')

        # Check each pair
        for pair in pairs:
            if not re.match(pair_pattern, pair):
                return False
        return True

    @staticmethod
    def verify_strings(string_array):
        results = {}
        for index, syntax in enumerate(string_array):
            results[index] = ConfigurationManager.is_valid_format(syntax)

        # Print results
        for key, value in results.items():
            print(f"{string_array[key]}: {'Valid' if value else 'Invalid'}")
            if not value:
                sys.exit()

    @staticmethod
    def extract_component_id(component):
        elements = component.split("_")
        element_type = elements[0]
        index_str = elements[1]
        if "-" in index_str:
            start_host = int(index_str.split("-")[0])
            end_host = int(index_str.split("-")[1]) + 1
            return list(range(start_host, end_host)), element_type
        else:
            return [int(index_str)], element_type

    @staticmethod
    def get_host(index, h_type, ubuntu_server, router_0, array_window_hosts, array_switches, array_dhcp_unikernel, array_ubuntu_base, array_ubuntu_intermediate, ubuntu_vc_server):
        h = ""
        if index == 0 and h_type == "u":
            h = ubuntu_server
        elif index == 0 and h_type == "r":
            h = router_0
        elif index == 0 and h_type == "uvcs":
            h = ubuntu_vc_server
        elif h_type == "pc":
            h = array_window_hosts[index]
        elif h_type == "sw":
            h = array_switches[index]
        elif h_type == "udh":
            h = array_dhcp_unikernel[index]
        elif h_type == "ub":
            h = array_ubuntu_base[index]
        elif h_type == "ui":
            h = array_ubuntu_intermediate[index]
        return h
