(* mirage >= 4.8.1 & < 4.9.0 *)
open Mirage

(* we need two network interfaces: a public side and a private side *)
(* a bit of magic: currently, multiple networks only work on Unix and Xen
   backends, so we can get away with this indexes-as-numbers-as-strings
   silliness.
   See https://github.com/mirage/mirage/issues/645 *)
let public_netif4 =
  Key.(if_impl is_solo5
         (netif ~group:"public4" "public4")
         (netif ~group:"public4" "0"))
let private_netif4 =
  Key.(if_impl is_solo5
         (netif ~group:"private4" "private4")
         (netif ~group:"private4" "1"))

let public_netif5 =
  Key.(if_impl is_solo5
         (netif ~group:"public5" "public5")
         (netif ~group:"public5" "0"))   
let private_netif5 =
  Key.(if_impl is_solo5
         (netif ~group:"private5" "private5")
         (netif ~group:"private5" "1"))

let public_netif6 =
  Key.(if_impl is_solo5
         (netif ~group:"public6" "public6")
         (netif ~group:"public6" "0"))
let private_netif6 =
  Key.(if_impl is_solo5
         (netif ~group:"private6" "private6")
         (netif ~group:"private6" "1"))

let public_netif7 =
  Key.(if_impl is_solo5
         (netif ~group:"public7" "public7")
         (netif ~group:"public7" "0"))
let private_netif7 =
  Key.(if_impl is_solo5
         (netif ~group:"private7" "private7")
         (netif ~group:"private7" "1"))
   
let public_netif8 =
  Key.(if_impl is_solo5
         (netif ~group:"public8" "public8")
         (netif ~group:"public8" "0"))
let private_netif8 =
  Key.(if_impl is_solo5
         (netif ~group:"private8" "private8")
         (netif ~group:"private8" "1"))
   
let public_netif9 =
  Key.(if_impl is_solo5
         (netif ~group:"public9" "public9")
         (netif ~group:"public9" "0"))
let private_netif9 =
  Key.(if_impl is_solo5
         (netif ~group:"private9" "private9")
         (netif ~group:"private9" "1"))
   
let public_netif10 =
  Key.(if_impl is_solo5
         (netif ~group:"public10" "public10")
         (netif ~group:"public10" "0"))
let private_netif10 =
  Key.(if_impl is_solo5
         (netif ~group:"private10" "private10")
         (netif ~group:"private10" "1"))
   
let public_netif11 =
  Key.(if_impl is_solo5
         (netif ~group:"public11" "public11")
         (netif ~group:"public11" "0"))
let private_netif11 =
  Key.(if_impl is_solo5
         (netif ~group:"private11" "private11")
         (netif ~group:"private11" "1"))
   
let public_netif12 =
  Key.(if_impl is_solo5
         (netif ~group:"public12" "public12")
         (netif ~group:"public12" "0"))
let private_netif12 =
  Key.(if_impl is_solo5
         (netif ~group:"private12" "private12")
         (netif ~group:"private12" "1"))
   
let public_netif13 =
  Key.(if_impl is_solo5
         (netif ~group:"public13" "public13")
         (netif ~group:"public13" "0"))
let private_netif13 =
  Key.(if_impl is_solo5
        (netif ~group:"private13" "private13")
        (netif ~group:"private13" "1"))

let packages = [
  package "ethernet";
  package "ipaddr";
  package ~min:"8.2.0" ~sublibs:["ipv4";"tcp"] "tcpip"; (* There was a breaking API changes in the 8.2 series *)
  package "re";
  package "logs";
  package ~min:"4.8.1" "mirage-runtime"; (* we need at least mirage 4.8.1 *)
]

(* our unikernel needs to know about physical network, and ethernet modules
   for each interface. Even though these modules will be the same for both
   interfaces in our case, we have to pass them separately. *)
let main = main "Unikernel.Main" ~packages
           (network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            network  @-> network  @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            ethernet @-> ethernet @->
            random   @-> mclock   @-> job)

(* we need to pass each of the network-related impls we've made to the
   unikernel, so that it can start the appropriate listeners. *)
let () = register "simple-ids" [ main
                              $ public_netif4          $ private_netif4
                              $ public_netif5          $ private_netif5
                              $ public_netif6          $ private_netif6
                              $ public_netif7          $ private_netif7
                              $ public_netif8          $ private_netif8
                              $ public_netif9          $ private_netif9
                              $ public_netif10         $ private_netif10
                              $ public_netif11         $ private_netif11
                              $ public_netif12         $ private_netif12
                              $ public_netif13         $ private_netif13
                              $ ethif public_netif4    $ ethif private_netif4
                              $ ethif public_netif5    $ ethif private_netif5
                              $ ethif public_netif6    $ ethif private_netif6
                              $ ethif public_netif7    $ ethif private_netif7
                              $ ethif public_netif8    $ ethif private_netif8
                              $ ethif public_netif9    $ ethif private_netif9
                              $ ethif public_netif10   $ ethif private_netif10
                              $ ethif public_netif11   $ ethif private_netif11
                              $ ethif public_netif12   $ ethif private_netif12
                              $ ethif public_netif13   $ ethif private_netif13
                              $ default_random  $ default_monotonic_clock
                              ]
