  let log = Logs.Src.create "ids" ~doc:"IDS rules"
  module Log = (val Logs.src_log log : Logs.LOG)

  (* Creates a set of rules (empty) and a default action (claim a packet is not an attack) *)
  let ids_rules =
    Rules.init false
    
  let attacker_ip_lst = ref []
  let node_id = ref 0

  (* Takes an IPv4 [packet], unmarshal it, examine to detect attackers' IPs in TCP packets*)
  let detect_and_output_private packet header frame inter_num output_private output_ether_private =
    (* Handle IPv4 only... *)
    match Ipv4_packet.Unmarshal.of_cstruct packet with
    | Result.Error s ->
      Logs.err (fun m -> m "Can't parse IPv4 packet: %s" s);
      Lwt.return_unit
    (* Otherwise unmarshal it, match rules with IPs, ports, proto *)
    | Result.Ok (ipv4_hdr, payload) ->
      (match Rules.is_match_rule ids_rules (ipv4_hdr, payload) with 
      | false -> 
          ()
      | true ->  
          attacker_ip_lst := ipv4_hdr.src :: !attacker_ip_lst;
          ());
      let options = ipv4_hdr.options in
      match List.exists (fun attacker_ip -> Ipaddr.V4.compare ipv4_hdr.dst attacker_ip = 0) !attacker_ip_lst with
      | false ->
        (* If the packet is NOT an attack, just look for registration request *)
        Custom_option.read_custom_opt ipv4_hdr.src options (!node_id, inter_num);
        output_private frame
      | true ->
        (* If the packet IS an attack, look for registration request AND embed reaction decision *)
        let new_options_cs = Custom_option.read_and_append_custom_opt ipv4_hdr.src ipv4_hdr.dst options (!node_id, inter_num) in
        let new_ipv4_hdr = {ipv4_hdr with options= new_options_cs} in
        let len_payload = Cstruct.length payload in
        let new_ipv4_hdr_cs = Ipv4_packet.Marshal.make_cstruct ~payload_len:len_payload new_ipv4_hdr in
        let new_packet = Cstruct.append new_ipv4_hdr_cs payload in
        output_ether_private header.Ethernet.Packet.source header.Ethernet.Packet.destination new_packet
