(* This unikernel is largely inspired by the example at https://github.com/mirage/mirage-nat/ *)

open Cmdliner

module Main
    (* our unikernel is functorized over the physical, and ethernet modules
       for the public and private interfaces, so each one shows up as a module
       argument. *)
      (Public_net4: Mirage_net.S) (Private_net4: Mirage_net.S)
      (Public_net5: Mirage_net.S) (Private_net5: Mirage_net.S)
      (Public_net6: Mirage_net.S) (Private_net6: Mirage_net.S)
      (Public_net7: Mirage_net.S) (Private_net7: Mirage_net.S)
      (Public_net8: Mirage_net.S) (Private_net8: Mirage_net.S)
      (Public_net9: Mirage_net.S) (Private_net9: Mirage_net.S)
      (Public_net10: Mirage_net.S) (Private_net10: Mirage_net.S)
      (Public_net11: Mirage_net.S) (Private_net11: Mirage_net.S)
      (Public_net12: Mirage_net.S) (Private_net12: Mirage_net.S)
      (Public_net13: Mirage_net.S) (Private_net13: Mirage_net.S)
      (Public_ethernet4 : Ethernet.S) (Private_ethernet4 : Ethernet.S)
      (Public_ethernet5 : Ethernet.S) (Private_ethernet5 : Ethernet.S)
      (Public_ethernet6 : Ethernet.S) (Private_ethernet6 : Ethernet.S)
      (Public_ethernet7 : Ethernet.S) (Private_ethernet7 : Ethernet.S)
      (Public_ethernet8 : Ethernet.S) (Private_ethernet8 : Ethernet.S)
      (Public_ethernet9 : Ethernet.S) (Private_ethernet9 : Ethernet.S)
      (Public_ethernet10 : Ethernet.S) (Private_ethernet10 : Ethernet.S)
      (Public_ethernet11 : Ethernet.S) (Private_ethernet11 : Ethernet.S)
      (Public_ethernet12 : Ethernet.S) (Private_ethernet12 : Ethernet.S)
      (Public_ethernet13 : Ethernet.S) (Private_ethernet13 : Ethernet.S)
      (Random : Mirage_crypto_rng_mirage.S) (Clock : Mirage_clock.MCLOCK)
  = struct

  module I4  = Listener.Make (Public_net4 ) (Private_net4 ) (Public_ethernet4 ) (Private_ethernet4 )
  module I5  = Listener.Make (Public_net5 ) (Private_net5 ) (Public_ethernet5 ) (Private_ethernet5 )
  module I6  = Listener.Make (Public_net6 ) (Private_net6 ) (Public_ethernet6 ) (Private_ethernet6 )
  module I7  = Listener.Make (Public_net7 ) (Private_net7 ) (Public_ethernet7 ) (Private_ethernet7 )
  module I8  = Listener.Make (Public_net8 ) (Private_net8 ) (Public_ethernet8 ) (Private_ethernet8 )
  module I9  = Listener.Make (Public_net9 ) (Private_net9 ) (Public_ethernet9 ) (Private_ethernet9 )
  module I10 = Listener.Make (Public_net10) (Private_net10) (Public_ethernet10) (Private_ethernet10)
  module I11 = Listener.Make (Public_net11) (Private_net11) (Public_ethernet11) (Private_ethernet11)
  module I12 = Listener.Make (Public_net12) (Private_net12) (Public_ethernet12) (Private_ethernet12)
  module I13 = Listener.Make (Public_net13) (Private_net13) (Public_ethernet13) (Private_ethernet13)


  (* configure logs, so we can use them later *)
  let log = Logs.Src.create "ids" ~doc:"IDS device"
  module Log = (val Logs.src_log log : Logs.LOG)

  let node_id =
    let doc = Arg.info ~doc:"Node ID" [ "node-id" ] in
    Mirage_runtime.register_arg Arg.(value & opt int32 5_000l doc)

  (* the specific impls we're using show up as arguments to start. *)
  let start public_netif4 private_netif4
            public_netif5 private_netif5
            public_netif6 private_netif6
            public_netif7 private_netif7
            public_netif8 private_netif8
            public_netif9 private_netif9
            public_netif10 private_netif10
            public_netif11 private_netif11
            public_netif12 private_netif12
            public_netif13 private_netif13
            _public_ethernet4 private_ethernet4
            _public_ethernet5 private_ethernet5
            _public_ethernet6 private_ethernet6
            _public_ethernet7 private_ethernet7
            _public_ethernet8 private_ethernet8
            _public_ethernet9 private_ethernet9
            _public_ethernet10 private_ethernet10
            _public_ethernet11 private_ethernet11
            _public_ethernet12 private_ethernet12
            _public_ethernet13 private_ethernet13
            _rng () =

    Ids.node_id := Int32.to_int (node_id ());

    (* start both listeners, and continue as long as both are working. *)
    Lwt.pick [
      I4.listen_public  public_netif4  private_netif4  private_ethernet4   3;
      I5.listen_public  public_netif5  private_netif5  private_ethernet5   4;
      I6.listen_public  public_netif6  private_netif6  private_ethernet6   5;
      I7.listen_public  public_netif7  private_netif7  private_ethernet7   6;
      I8.listen_public  public_netif8  private_netif8  private_ethernet8   7;
      I9.listen_public  public_netif9  private_netif9  private_ethernet9   8;
      I10.listen_public public_netif10 private_netif10 private_ethernet10  9;
      I11.listen_public public_netif11 private_netif11 private_ethernet11 10;
      I12.listen_public public_netif12 private_netif12 private_ethernet12 11;
      I13.listen_public public_netif13 private_netif13 private_ethernet13 12;

      I4.listen_private  public_netif4  private_netif4  ;
      I5.listen_private  public_netif5  private_netif5  ;
      I6.listen_private  public_netif6  private_netif6  ;
      I7.listen_private  public_netif7  private_netif7  ;
      I8.listen_private  public_netif8  private_netif8  ;
      I9.listen_private  public_netif9  private_netif9  ;
      I10.listen_private public_netif10 private_netif10 ;
      I11.listen_private public_netif11 private_netif11 ;
      I12.listen_private public_netif12 private_netif12 ;
      I13.listen_private public_netif13 private_netif13 ;
    ]
end
