# from scapy.all import *
# from scapy.layers.inet import IP
# from scapy.layers.smb import SMB_Header
import json
import socket
import subprocess
import threading
import time

import networkx as nx
import numpy as np
import topohub
import struct

import gns3manager
# import matplotlib.pyplot as plt
from gns3manager import *

DIR = "/home/anh/Documents/kernel_preparation/gns3_unikernel_testbed"

NUM_SWITCH = 0  # Number of switches
NUM_HOST = 0
NUM_UBUNTU_BASE = 0
NUM_UBUNTU_INTERMEDIATE = 0
NUM_UBUNTU_UNCTRL = 0


#
# def process_packet(pkt):
#     global array_point, continue_sniffing, malware_instance, is_infected, map_infecting, map_infector
#     if SMB_Header in pkt:
#         if not is_infected and pkt[SMB_Header].Command == 0xa0 and pkt[SMB_Header].Flags == 0x18:
#             victim_ip = pkt[IP].dst
#             infecting_ip = pkt[IP].src
#             if victim_ip not in map_infecting.keys():
#                 map_infecting[victim_ip] = [infecting_ip]
#             elif infecting_ip not in map_infecting[victim_ip]:
#                 map_infecting[victim_ip].append(infecting_ip)
#             output_file = "infecting.txt"
#             with open(output_file, "w") as f:
#                 json.dump(map_infecting, f)
#
#         if (malware_instance == 0 and pkt[SMB_Header].MID == 81) or (
#                 malware_instance == 1 and pkt[SMB_Header].Reserved == 0x0011):
#             array_point.append(pkt[IP].src + "," + pkt[IP].dst)
#             if pkt[IP].src not in map_infector:
#                 map_infector[pkt[IP].src] = 1
#             else:
#                 input("Error" + pkt[IP].src)
#             print("saved", pkt[IP].src, " - ", pkt[IP].dst, "- size", len(array_point))
#
#             output_file = "infector.txt"
#             with open(output_file, "w") as f:
#                 f.write(str(array_point))
#             continue_sniffing = False
#             is_infected = True
#
#

def send_para_server(A_raw, f, num_host, p_NAT, P_values, max_total_traffic, proto, time_generate, pref_ctrl_as_percen):
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
        server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        server_socket.bind(('0.0.0.0', 12348))
        server_socket.listen()
        print("Server started on 12348")

        while True:
            client_socket, client_address = server_socket.accept()
            with client_socket:  # Ensures cleanup
                data = {'A_raw': A_raw, 'f': f, 'num_host': num_host, 'p_NAT': p_NAT, 'P_values': P_values, 'max_total_traffic': max_total_traffic, 'proto': proto,
                        'time_generate': time_generate, 'pref_ctrl_as_percen': pref_ctrl_as_percen}
                data_js = json.dumps(data)
                client_socket.sendall(data_js.encode('utf-8'))  # Send JSON data
                break
        client_socket.close()
    server_socket.close()


def run_vc_server(map_connection_ubuntu_base, is_reaction):
    global NUM_UBUNTU_BASE
    map_json = json.dumps({"map_ip": map_connection_ubuntu_base, "is_reaction": is_reaction, "total_router": NUM_UBUNTU_BASE})
    command_str = f"python3 {DIR}/ubuntu_base/vc_server.py '{map_json}'"
    subprocess.Popen(
        ['xterm', '-hold', '-e', command_str],
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
        stdin=subprocess.DEVNULL,
        start_new_session=True
    )


def run_unctrl_server(map_connection_ubuntu_unctrl):
    global NUM_UBUNTU_UNCTRL
    unctrl_id = 0
    listen_port = 12347
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
        server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        server_socket.bind(('0.0.0.0', listen_port))
        server_socket.listen()
        while True:
            client_socket, client_address = server_socket.accept()
            data = client_socket.recv(6)  # Expecting 6 byte
            if data:
                (mac_bytes,) = struct.unpack('!6s', data)
                unctrl_id += 1
                id_to_send = unctrl_id
                mac_address = ':'.join(f'{b:02x}' for b in mac_bytes)
                node_param = {"ip_addresses": map_connection_ubuntu_unctrl[mac_address], "total_router": NUM_UBUNTU_UNCTRL}
                json_data = json.dumps(node_param).encode()
                json_size = len(json_data)
                packed_data = struct.pack('!II', id_to_send, json_size) + json_data
                client_socket.sendall(packed_data)
                client_socket.close()

            if unctrl_id == NUM_UBUNTU_UNCTRL:
                break
    server_socket.close()


def setup_transit_AS(array_ubuntu_router, array_switches, array_ubuntu_intermediate, topohub_graph, ip_as, ip_user, num_ubuntu):
    # Link all u_router to sw
    for i in range(num_ubuntu):
        ur = array_ubuntu_router[i]
        array_switches[0].connect(ur)

    # Link u_router together
    ur_connections = topohub_graph.edges(data=True)
    map_connection = {}
    start_2nd_octet = 100
    start_3nd_octet = 0
    for conn in ur_connections:
        node_index1 = int(conn[0])
        node_index2 = int(conn[1])

        ur_1 = array_ubuntu_router[node_index1]
        ur_2 = array_ubuntu_router[node_index2]

        ur_1_port, ur_1_adapter, ur_2_port, ur_2_adapter = ur_1.connect(ur_2)

        node_1_js = GNS3Manager.get_node(ur_1.id)
        node_2_js = GNS3Manager.get_node(ur_2.id)

        if start_3nd_octet == 255 and start_2nd_octet < 255:
            start_3nd_octet = 0
            start_2nd_octet += 1
        else:
            start_3nd_octet += 1

        node_1_ip = ".".join([ip_as, str(start_2nd_octet), str(start_3nd_octet), "254"])
        node_2_ip = ".".join([ip_as, str(start_2nd_octet), str(start_3nd_octet), "253"])

        if node_1_js["ports"][0]["mac_address"] not in map_connection:
            map_connection[node_1_js["ports"][0]["mac_address"]] = {}
        map_connection[node_1_js["ports"][0]["mac_address"]][ur_1_adapter] = node_1_ip

        if node_2_js["ports"][0]["mac_address"] not in map_connection:
            map_connection[node_2_js["ports"][0]["mac_address"]] = {}
        map_connection[node_2_js["ports"][0]["mac_address"]][ur_2_adapter] = node_2_ip

    # Link u_router to ui respectively
    start_2nd_octet = 100
    start_3nd_octet = 0
    for i in range(num_ubuntu):
        ur = array_ubuntu_router[i]
        ui = array_ubuntu_intermediate[i]

        ur_port, ur_adapter, ui_port, ui_adapter = ur.connect(ui)

        ur_js = GNS3Manager.get_node(ur.id)

        if start_3nd_octet == 255 and start_2nd_octet < 255:
            start_3nd_octet = 0
            start_2nd_octet += 1
        else:
            start_3nd_octet += 1

        ur_ip = ".".join([ip_user, str(start_2nd_octet), str(start_3nd_octet), "254"])

        if ur_js["ports"][0]["mac_address"] not in map_connection:
            map_connection[ur_js["ports"][0]["mac_address"]] = {}
        map_connection[ur_js["ports"][0]["mac_address"]][ur_adapter] = ur_ip

    return map_connection


def generate_topohub_GNS3_topology():
    global NUM_SWITCH, NUM_HOST, NUM_UBUNTU_BASE, NUM_UBUNTU_INTERMEDIATE, NUM_UBUNTU_UNCTRL

    gns3manager.PROJECT_ID, project_name = GNS3Manager.get_projectId()
    user_input = input("Press enter if correct project to generate topology?")
    if user_input != '':
        sys.exit()

    if len(sys.argv) > 10:
        topology_name = sys.argv[1]
        gns3manager.malware_instance = 0 if sys.argv[2].strip() == "wannacry" else 1
        f = float(sys.argv[3])
        p_NAT = int(sys.argv[4])
        packet_lost = int(sys.argv[5])
        max_total_traffic = int(sys.argv[6])
        proto = sys.argv[7]
        time_generate = int(sys.argv[8])
        pref_ctrl_as_percen = int(sys.argv[9])
        is_reaction = int(sys.argv[10])

        topo = topohub.get(topology_name)
        g = nx.node_link_graph(topo)

        P_values = np.random.lognormal(mean=0, sigma=1, size=g.graph['stats']['nodes']).tolist()
        # P_values = [0.49729683886415604, 2.8478372536019494, 0.5638497692033089, 1.7066441178756397, 1.0955777113276581, 0.6624609705231256, 1.2170507302967504, 2.241922851235645,
        #             0.9904057794366734, 0.10045359874192523, 0.1331217360266883, 0.7161751639559052]
        print("P_values:", P_values)

        # Extract X (longitude) and Y (latitude) positions
        positions = {node: data["pos"] for node, data in g.nodes(data=True)}

        # Convert positions to a NumPy array for easier manipulation
        pos_array = np.array(list(positions.values()))

        # Compute centroids (mean x and y)
        center_x = np.mean(pos_array[:, 0])
        center_y = np.mean(pos_array[:, 1])

        # Shift all positions so the mean is at (0,0)
        shifted_positions = {
            node: [data["pos"][0] - center_x, data["pos"][1] - center_y]
            for node, data in g.nodes(data=True)
        }

        # Now, let's find the new min and max values after centering
        shifted_pos_array = np.array(list(shifted_positions.values()))
        min_x, min_y = shifted_pos_array.min(axis=0)
        max_x, max_y = shifted_pos_array.max(axis=0)

        # Scale the positions so that they fit into the 0-1000 range (in GNS3)
        scaled_positions = {
            node: [
                # Scale X from -max_x to max_x into 0 to 2000
                float(1000 * (data[0] / max(abs(min_x), abs(max_x)))),  # Longitude (X)

                # Scale Y from -max_y to max_y into 0 to 1400, invert Y-axis to fit GNS3 convention
                float(1400 - (500 * (data[1] / max(abs(min_y), abs(max_y)))) - 1000)  # Latitude (Y) inverted
            ]
            for node, data in shifted_positions.items()
        }

        # Update graph with new positions
        nx.set_node_attributes(g, scaled_positions, "pos")

        # Access graph parameters
        print(g.graph['name'])
        # print(g.graph['demands'])
        print(g.graph['stats']['nodes'])
        topology_nodes = g.nodes(data=True)
        # print(topology_nodes)

        NUM_SWITCH = 1  # Number of switches
        NUM_HOST = g.graph['stats']['nodes']  # Number of PCs
        NUM_UBUNTU_BASE = g.graph['stats']['nodes']
        NUM_UBUNTU_INTERMEDIATE = g.graph['stats']['nodes']
        NUM_UBUNTU_UNCTRL = g.graph['stats']['nodes']

        print("Number of switches:", NUM_SWITCH)
        print("Number of PCs:", NUM_HOST)
        print("Number of ubuntu base:", NUM_UBUNTU_BASE)
        print("Number of ubuntu intermediate:", NUM_UBUNTU_INTERMEDIATE)

        #     mapping_ip_connections = {}
        #     with open(sys.argv[3], 'r') as file:
        #         for line in file:
        #             key, value = line.strip().split(":")
        #             mapping_ip_connections[key] = value
        #
        array_existing_nodes = GNS3Manager.get_nodes()
        array_window_hosts = []
        array_switches = []
        array_ubuntu_base = [Node(0, 0, 0, 0)] * NUM_UBUNTU_BASE
        array_ubuntu_intermediate = []
        ubuntu_server = Node(1, 1, GNS3Manager.create_ubuntu_server(1200, 1000), UBUNTU_TYPE)
        ubuntu_vc_server = Node(1, 1, "", UBUNTU_VC_SERVER_TYPE)
        array_ubuntu_unctrl = [Node(0, 0, 0, 0)] * NUM_UBUNTU_UNCTRL

        for exist_node in array_existing_nodes:
            if exist_node[1] == "NAT1":
                ubuntu_vc_server.id = exist_node[0]
                x_node = exist_node[2]
                y_node = exist_node[3]
                z_node = exist_node[4]

        # create ubuntu base
        for key, value in topology_nodes:
            node_index = int(key)
            x_temp = round(value["pos"][0])
            y_temp = round(value["pos"][1] + 1000)
            array_ubuntu_base[node_index] = (Node(1, MAX_SIZE_OF_UBUNTU_BASE_PORT, GNS3Manager.create_ubuntu_base(x_temp, y_temp), UBUNTU_BASE_TYPE))

        # create ubuntu unctrl
        for key, value in topology_nodes:
            node_index = int(key)
            x_temp = round(value["pos"][0])
            y_temp = round(value["pos"][1] - 1000)
            array_ubuntu_unctrl[node_index] = (Node(1, MAX_SIZE_OF_UBUNTU_BASE_PORT, GNS3Manager.create_ubuntu_unctrl(x_temp, y_temp), UBUNTU_UNCTRL_TYPE))

        # create ubuntu intermediate
        for i in range(NUM_UBUNTU_INTERMEDIATE):
            x_temp = round(1200)
            y_temp = round(-1000) + 100 * i
            array_ubuntu_intermediate.append(Node(1, MAX_SIZE_OF_UBUNTU_BASE_PORT, GNS3Manager.create_ubuntu_intermediate(x_temp, y_temp), UBUNTU_INTERMEDIATE_TYPE))

        # Create Windows hosts
        if NUM_HOST > 0:
            for i in range(0, NUM_HOST):
                is_attacker = 0
                if i == 0:
                    is_attacker = 1
                x_temp = round(1400)
                y_temp = round(-1000) + 100 * i
                array_window_hosts.append(Node(1, 1, GNS3Manager.create_window(x_temp, y_temp, is_attacker), WINDOW_TYPE))

        # Create switches
        x_local = -1200
        y_local = -1000
        step = 100
        for i in range(NUM_SWITCH):
            x_temp = x_local
            y_temp = y_local + step * i
            array_switches.append(Node(MAX_SIZE_OF_SWITCH_PORT, 1, GNS3Manager.create_switch(MAX_SIZE_OF_SWITCH_PORT, x_temp, y_temp, 1), SWITCH_TYPE))

        # Link the VC server host to sw
        array_switches[0].connect(ubuntu_vc_server)

        # Link ub
        map_connection_ubuntu_base = setup_transit_AS(array_ubuntu_base, array_switches, array_ubuntu_intermediate, g, "192", "123", NUM_UBUNTU_BASE)

        # Link ub_0 to ubuntu_server
        ub_0 = array_ubuntu_base[0]
        ub_0_port, ub_0_adapter, ubuntu_server_port, ubuntu_server_adapter = ub_0.connect(ubuntu_server)
        ub_0_js = GNS3Manager.get_node(ub_0.id)
        ub_0_ip = "10.1.1.254"
        if ub_0_js["ports"][0]["mac_address"] not in map_connection_ubuntu_base:
            map_connection_ubuntu_base[ub_0_js["ports"][0]["mac_address"]] = {}
        map_connection_ubuntu_base[ub_0_js["ports"][0]["mac_address"]][ub_0_adapter] = ub_0_ip

        # Link windows to ui respectively
        for i in range(NUM_HOST):
            pc = array_window_hosts[i]
            ui = array_ubuntu_intermediate[i]
            pc.connect(ui)

        # Link unctrl
        map_connection_ubuntu_unctrl = setup_transit_AS(array_ubuntu_unctrl, array_switches, array_ubuntu_intermediate, g, "172", "124", NUM_UBUNTU_UNCTRL)

        # if input("Start Ubuntu VC Server (y/N)?") != "y":
        #     sys.exit()
        threading.Thread(target=run_vc_server, args=[map_connection_ubuntu_base, is_reaction], daemon=True).start()
        threading.Thread(target=run_unctrl_server, args=[map_connection_ubuntu_unctrl], daemon=True).start()

        # Press y to start Ubuntu bases
        if input("Start Ubuntu Router nodes (y/N)?") != "y":
            sys.exit()
        GNS3Manager.start_nodes(array_ubuntu_unctrl, "unctrl_r")
        GNS3Manager.start_nodes(array_ubuntu_base, "ubase_r")
        GNS3Manager.start_nodes(array_ubuntu_intermediate, "uinter_r")

        # Enter y to start router R1 and ubuntu server
        if input("Start Ubuntu Server (y/N)?") != "y":
            sys.exit()
        GNS3Manager.start_node(ubuntu_server.id, "u_server")

        send_para_server(g.graph['demands'], f, NUM_HOST, p_NAT, P_values, max_total_traffic, proto, time_generate, pref_ctrl_as_percen)

        array_links = GNS3Manager.get_links()
        for link in array_links:
            for unctrl_host in array_ubuntu_unctrl:
                if unctrl_host.id in link and array_switches[0].id in link:
                    GNS3Manager.del_link(link[0])

        # Capture packets from communications/links
        if input("Capture link (y/N)?") == "y":
            for link in array_links:
                for window_host in array_window_hosts:
                    if window_host.id in link:
                        GNS3Manager.start_capture_link(link[0])

                for ubuntu_base in array_ubuntu_base:
                    if ubuntu_base.id in link:
                        GNS3Manager.set_link_filter(link[0], packet_lost)

        # Press y to start Windows PCs
        if input("Start Windows PCs (y/N)?") != "y":
            sys.exit()
        GNS3Manager.start_nodes(array_window_hosts, "win")

        # TCP listen on port 12348 to collect propagation time data
        command = "nc -l -p 12348 > measurement.txt"
        try:
            subprocess.run(command, shell=True, check=True)
            print("Command executed successfully.")
            GNS3Manager.nocapture()  # Stop capturing packets after receiving data
            # Store data in a .zip
            subprocess.run("zip a.zip /mnt/raid5/projects/" + PROJECT_ID + "/project-files/captures/*", shell=True, check=True)
        except subprocess.CalledProcessError as e:
            print(f"Error executing command: {e}")


if __name__ == "__main__":
    generate_topohub_GNS3_topology()
