from bcc import BPF
from bcc.utils import printb
import time
import socket
import struct

device = "vmnet8"
b = BPF(src_file="op_ebpf.c")
fn = b.load_func("op_ebpf", BPF.XDP)
b.attach_xdp(device, fn, 0)

try:
    # b.trace_print()
    queue = b[b'queue']

    while (True):
        try:
            req = queue.pop()
            # Unpack the struct (assuming 3 x __u32 values)
            entry_inter_val, ip_dst, entry_cmd_val = struct.unpack("III", req)

            ip_dst_le = struct.unpack("<I", struct.pack(">I", ip_dst))[0]  # Convert to little-endian
            # Convert the IP from 32-bit integer (now little-endian) to string
            ip_str = socket.inet_ntoa(struct.pack("!I", ip_dst_le)) 

            # Print the extracted values
            print(f"Entry: {entry_inter_val}, IP: {ip_str}, Cmd: {entry_cmd_val}")

            # dist = b.get_table("counter")
            
            # for k, v in sorted(dist.items(), key=lambda x: x[0].value):
                # print("DEST_PORT : %10d, COUNT : %10d" % (k.value, v.value))
            time.sleep(10)
        except KeyError:
            # If the queue is empty, continue
            continue
except KeyboardInterrupt:
    dist = b.get_table("counter")

    # Use k.value to extract the integer value for sorting
    for k, v in sorted(dist.items(), key=lambda x: x[0].value):
        print("DEST_PORT : %10d, COUNT : %10d" % (k.value, v.value))

b.remove_xdp(device, 0)
