# from netfilterqueue import NetfilterQueue
from opportunistic_utils import *
from scapy.layers.inet import IP
import if_utils
# import ctypes

from bcc import BPF
# from bcc.utils import printb
import time
import socket
import struct


# event_queue_firewall = if_utils.EventDrivenQueue(setup_firewall)

# def process_packet(new_packet):
#     inter_num = new_packet.get_mark()
#     # Determine the incoming interface based on the mark
#     # interface = MARK_TO_INTERFACE.get(inter_num, 'unknown')
#     # print(interface)
#
#     data = new_packet.get_payload()
#
#     pkt = IP(data)
#
#     # Parse the raw data into a Scapy IP packet
#     # pkt = IP(data)
#
#     if pkt.haslayer(IP):
#         # print(f"IP Packet: {pkt.summary()}")
#         # pkt.show()
#
#         is_register_me = True
#         index_custom_option = -1
#
#         if pkt.options:
#             for i in range(len(pkt.options)):
#                 option = pkt.options[i]
#                 if isinstance(option, CustomIPOption) and option.option == MY_OPTION_TYPE:
#                     index_custom_option = i
#                     for entry in option.read_all_entry():
#                         if entry["cmd"] == Cmd.DECISION_DROP:
#                             # Do something to deploy firewall if the decision is for me
#                             if entry["node_id"] == if_utils.my_id:
#                                 print(f"Custom Option from DECISION found: id = {entry['node_id']}, inter = {entry['inter_num']}, cmd = {entry['cmd']}")
#                                 # event_queue_firewall.put((entry['inter_num'], pkt[IP].dst, entry['cmd']))
#                                 FirewallOpportunistic.firewall_queue.put((entry['inter_num'], pkt[IP].dst, entry['cmd']))
#                         elif entry["cmd"] == Cmd.REGISTER:
#                             is_register_me = False
#
#         if is_register_me:
#             if index_custom_option == -1:
#                 option = CustomIPOption()
#                 option.add_entry(if_utils.my_id, inter_num, Cmd.REGISTER)
#                 if pkt.options:
#                     pkt.options.append(option)  # Append to the existing options
#                 else:
#                     pkt.options = [option]  # Add as the first option if none exist
#             else:
#                 pkt.options[index_custom_option].add_entry(if_utils.my_id, inter_num, Cmd.REGISTER)
#
#             del pkt.len
#             del pkt.ihl
#             del pkt[IP].chksum  # Invalidate the checksum
#             new_packet.set_payload(bytes(pkt))
#             # pkt.show()  # Now this will work correctly
#
#     new_packet.accept()


def check_local_packet(pkt):
    if pkt.haslayer(IP) and pkt.options:
        for i in range(len(pkt.options)):
            option = pkt.options[i]
            if isinstance(option, CustomIPOption) and option.option == MY_OPTION_TYPE:
                for entry in option.read_all_entry():
                    if entry["cmd"] == Cmd.DECISION_DROP:
                        # Do something to deploy firewall if the decision is for me
                        if entry["node_id"] == if_utils.my_id:
                            # print(f"Custom Option from DECISION found: id = {entry['node_id']}, inter = {entry['inter_num']}, cmd = {entry['cmd']}")
                            FirewallOpportunistic.firewall_queue.put((entry['inter_num'], pkt[IP].dst, entry['cmd']))


def process_opportunistic_node():
    # # Insert ebtables rule to drop VC messages to avoid broadcast neighbors' traffic
    if_utils.drop_layer2_VC_packet_rule("00:00:00:00:00:00")
    for inter_num in range(if_utils.MIN_INTER_NUMBER, if_utils.MAX_INTER_NUMBER):
        if_utils.map_bridge_outside[inter_num] = None
        inter_name = if_utils.PHYSICAL_INTERFACE[inter_num]
        if_utils.map_inf_inside[inter_num] = inter_name


def modify_node_id(file_path, new_value):
    with open(file_path, 'r+') as file:
        lines = file.readlines()  # Read all lines
        for i, line in enumerate(lines):
            if line.startswith("#define MY_NODE_ID"):
                lines[i] = f"#define MY_NODE_ID {new_value}\n"
                break  # Exit the loop once we've made the change
        file.seek(0)
        file.writelines(lines)


def start_process_from_queue():
    file_path = if_utils.DIR + '/ubuntu_base/op_ebpf.c'  # Replace with your C file path
    modify_node_id(file_path, if_utils.my_id)

    b = BPF(src_file=f"op_ebpf.c")
    fn = b.load_func("op_ebpf", BPF.XDP)
    for inter_num in range(if_utils.MIN_INTER_NUMBER, if_utils.MAX_INTER_NUMBER):
        inter_name = if_utils.PHYSICAL_INTERFACE[inter_num]
        b.attach_xdp(inter_name, fn, 0)
        print("Attach the ebpf program at", inter_name)

    try:
        request_queue = b[b'queue']

        while True:
            try:
                req = request_queue.pop()
                entry_inter_val, ip_dst, entry_cmd_val = struct.unpack("III", req)
                ip_dst_le = struct.unpack("<I", struct.pack(">I", ip_dst))[0]  # Convert to little-endian
                ip_str = socket.inet_ntoa(struct.pack("!I", ip_dst_le))

                # print(f"Entry: {entry_inter_val}, IP: {ip_str}, Cmd: {entry_cmd_val}")
                FirewallOpportunistic.firewall_queue.put((entry_inter_val, ip_str, entry_cmd_val))
                # time.sleep(1)
            except KeyError:
                continue
    except Exception:
        print("Interrupted")

    for inter_num in range(if_utils.MIN_INTER_NUMBER, if_utils.MAX_INTER_NUMBER):
        inter_name = if_utils.PHYSICAL_INTERFACE[inter_num]
        b.remove_xdp(inter_name, 0)


if __name__ == "__main__":
    input_id = int(sys.argv[1])
    # Enable netfilter on bridges, create br0
    if_utils.enable_br_netfilter()
    if_utils.create_bridge(if_utils.BR_CENTER_NAME)
    process_opportunistic_node(input_id)
    start_process_from_queue()

# TODO: When considering dynamic topology, broadcast communication must be checked if it can forwarded after nfqueue
# TODO: IDS must forward crafted packet from fw to nfqueue br0 in case fw - ids - br0
