import socket, struct
import threading
import time
import json
import sys

# Configuration
LISTEN_PORT = 12346  # Specify the port number to listen on
node_index_id = 0
map_security_node = {}
time_converge = 0


class State:
    INIT = 0
    COMPUTING = 1
    DECIDED_IN_VC = 2
    DECIDED_NOT_VC = 3


def listen_for_node(id_node, client_socket):
    while True:
        data = client_socket.recv(8)  # Expecting 1 byte
        if data:
            state, pointer = struct.unpack('!II', data)
            # print(f"Received in id: {id_node} - state: {state} - pointer: {pointer}")

            if map_security_node[id_node]["state"] != state or map_security_node[id_node]["pointer"] != pointer:
                map_security_node[id_node]["state"] = state
                map_security_node[id_node]["pointer"] = pointer

                # client_socket.close()


def print_mapping_state():
    global map_security_node, time_converge
    old_map_security = {}
    change_count = 0
    is_converged = False

    while True:
        try:
            contents = ["--- New record ---"]
            is_changed = False
            for id_node in map_security_node.keys():
                node = map_security_node[id_node]
                contents.append(f"Node {id_node:<5} "
                                f"| State: {'IN VC' if node['state'] == State.DECIDED_IN_VC else 'NOT in VC' if node['state'] == State.DECIDED_NOT_VC else 'None':<10} "
                                f"| Pointer: {node['pointer']}")
                if id_node not in old_map_security:
                    old_map_security[id_node] = {"state": node['state'], "pointer": node['pointer'], "socket": node['socket']}
                elif old_map_security[id_node]['state'] != map_security_node[id_node]['state'] or old_map_security[id_node]['pointer'] != map_security_node[id_node]['pointer']:
                    old_map_security[id_node]['state'] = node['state']
                    old_map_security[id_node]['pointer'] = node['pointer']
                    is_changed = True
            if is_changed:
                old_time_converge = time_converge
                time_converge = time.time()
                is_converged = False
                for cont in contents:
                    print(cont)
                change_count += 1
                print("Round (Num of change): ", change_count)
                if old_time_converge != 0:
                    print("Round time: ", time_converge - old_time_converge)
            elif time_converge != 0 and not is_converged and (time.time() - time_converge) > 10:
                is_converged = True
                print("Converged")
        except RuntimeError:
            pass


def main():
    global node_index_id, map_security_node

    params = json.loads(sys.argv[1])
    map_ip = params["map_ip"]
    is_reaction = params["is_reaction"]
    total_router = params["total_router"]

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
        server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        server_socket.bind(('0.0.0.0', LISTEN_PORT))
        server_socket.listen()

        print(f"Listening for incoming connections on port {LISTEN_PORT}...")

        threading.Thread(target=print_mapping_state).start()

        while True:
            client_socket, client_address = server_socket.accept()
            data = client_socket.recv(14)  # Expecting 14 byte
            if data:
                # Unpack the boolean and integer
                state, pointer, mac_bytes = struct.unpack('!II6s', data)

                if state == State.INIT:
                    print(f"Received state: {state}, pointer: {pointer}")

                    node_index_id += 1
                    id_to_send = node_index_id
                    map_security_node[node_index_id] = {"state": state, "pointer": pointer, "socket": client_socket}

                    mac_address = ':'.join(f'{b:02x}' for b in mac_bytes)
                    vc_node_param = {"ip_addresses": map_ip[mac_address], "is_reaction": is_reaction, "total_router": total_router}

                    json_data = json.dumps(vc_node_param).encode()
                    json_size = len(json_data)
                    packed_data = struct.pack('!II', id_to_send, json_size) + json_data

                    threading.Thread(target=listen_for_node, args=(id_to_send, client_socket)).start()

                    client_socket.sendall(packed_data)


if __name__ == "__main__":
    main()
