import socket, sys
import struct
import time
import json
import subprocess
from scapy.all import get_if_hwaddr

from pathlib import Path

sys.path.append(str(Path(__file__).parent.parent.resolve()))
from ubuntu_base import if_utils

LISTEN_PORT = 12347  # Specify the port number to listen on
client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

while True:
    try:
        client_socket.connect(('192.168.122.1', LISTEN_PORT))
        break  # Exit loop once connected
    except (socket.error, ConnectionRefusedError) as e:
        time.sleep(2)  # Wait before retrying


def update_ospf_rule(input_map, total_router):
    try:
        MY_BGP_AS_NUM = 65124
        subnets = []
        subnets_user = []
        ip_infs_user = []
        infs_user = []

        commands = "configure terminal"
        commands += "\nrouter ospf"
        commands += f"\npassive-interface ens3"

        bgp_commands = f"\nrouter bgp {MY_BGP_AS_NUM}"

        lo_ip = f"3.3.{if_utils.my_id}.3"
        bgp_commands += f"\nbgp router-id {lo_ip}"
        if_utils.add_ip_address("lo", lo_ip, "/32")

        for iface_number_str, ip in input_map.items():
            iface_name = f'ens{int(iface_number_str) + 3}'  # setup dhcp on ens inf, no need to set up on tap infs
            octets = ip.split(".")
            octets[-1] = "0"
            subnet = ".".join(octets)
            subnets.append(subnet)

            # Check if the ip address belongs to users to run dhcp server
            if subnet.startswith('124'):
                subnets_user.append(subnet)
                ip_infs_user.append(ip)
                infs_user.append(iface_name)
                commands += f"\npassive-interface {iface_name}"

                temp_octets = ip.split(".")
                temp_octets[-1] = "1"
                ip_peer = ".".join(temp_octets)
                bgp_commands += f"\nneighbor {ip_peer} remote-as 1{octets[2]}"

        for i in range(1, total_router + 1):
            bgp_commands += f"\nneighbor 3.3.{i}.3 remote-as {MY_BGP_AS_NUM}"
            bgp_commands += f"\nneighbor 3.3.{i}.3 update-source lo"

        if_utils.setup_dhcp(infs_user, ip_infs_user, subnets_user)

        for subnet in subnets:
            commands += f"\nnetwork {subnet}/24 area 0"
        commands += f"\nnetwork {lo_ip}/32 area 0"

        commands += "\nexit"
        # bgp_commands += "\nredistribute ospf"
        commands += bgp_commands
        commands += "\nexit\nexit\nwrite"

        # Start vtysh in interactive mode
        process = subprocess.Popen(
            ["sudo", "vtysh"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            universal_newlines=True,
        )
        output, errors = process.communicate(input=commands)
        if process.returncode == 0:
            print("OSPF configuration completed successfully.")
        else:
            print("Failed to configure OSPF.")

    except Exception as e:
        print(f"An error occurred: {e}")


def setup_network():
    global client_socket
    mac_address = get_if_hwaddr("ens3")  # Example: "00:1A:2B:3C:4D:5E"
    mac_bytes = bytes.fromhex(mac_address.replace(":", ""))
    message = struct.pack('!6s', mac_bytes)
    client_socket.sendall(message)

    response = client_socket.recv(8)
    received_id, json_size = struct.unpack('!II', response)
    json_data = client_socket.recv(json_size).decode()
    params = json.loads(json_data)
    data_map = params["ip_addresses"]
    total_router = params["total_router"]

    print(f"Received ID: {received_id}")
    print(f"Received Map: {data_map}")

    if_utils.my_id = received_id

    for iface_number_str, ip in data_map.items():
        inter_num = int(iface_number_str) + if_utils.MIN_INTER_NUMBER - 1
        iface_name = f'{if_utils.NAME_INF}{inter_num + 1}'
        if_utils.disable_NetworkManager_auto_ip(iface_name)
        if_utils.add_ip_address(iface_name, ip, "/24")

    update_ospf_rule(data_map, total_router)


def main():
    if_utils.wait_for_ip("ens3", "192.168.122")
    setup_network()


if __name__ == "__main__":
    main()
