import socket
import time, sys, queue
import threading
import json

# Configuration
LISTEN_PORT = 12346  # Specify the port number to listen on
ip_queue = queue.Queue()
deploy_notif_queue = queue.Queue()
array_point_save = []
array_deploy_notif_save = []
num_machine = 0
map_point = {}
OUTPUT_FILE = "measurement.txt"
DEPLOY_NOTIF_FILE = "deploy_notif.txt"
start_time = 0


def read_and_write_points():
    global ip_queue, array_point_save, map_point, num_machine, start_time
    while True:
        try:
            time_ip = ip_queue.get()
            if time_ip[1] not in map_point:
                map_point[time_ip[1]] = time_ip[0]
                array_point_save.append(time_ip)
                with open(OUTPUT_FILE, "w") as file:
                    file.write(str(array_point_save))
                len_actual = len(array_point_save)
                if start_time == 0:
                    start_time = time_ip[0]
                print(f"PC {len_actual} is infected at", time_ip[0] - start_time, "(s)")
            else:
                print("duplicate ", array_point_save)
        except queue.Empty:
            pass


def read_and_write_deploy_notif():
    global deploy_notif_queue, num_machine, start_time
    while True:
        try:
            deploy_time, data = deploy_notif_queue.get()
            try:
                data_js = json.loads(data.decode('utf-8'))
                node_id = data_js['node_id']
                ip_att = data_js['ip_att']
                array_deploy_notif_save.append([node_id, ip_att, deploy_time])

                with open(DEPLOY_NOTIF_FILE, "w") as file:
                    file.write(str(array_deploy_notif_save))

                print(f"Node {node_id} stopped", ip_att, f"at {deploy_time - start_time}", "(s)")
            except json.JSONDecodeError:
                print("Invalid JSON received")
        except queue.Empty:
            pass


def get_deploy_notif():
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
        server_socket.bind(('0.0.0.0', 12344))
        server_socket.listen()
        print("Server started on 12344")

        while True:
            client_socket, client_address = server_socket.accept()
            data = client_socket.recv(1024)
            deploy_notif_queue.put([time.time(), data])
            client_socket.close()


def main():
    global ip_queue, num_machine

    if len(sys.argv) > 1:
        num_machine = int(sys.argv[1])
    else:
        return

    thread = threading.Thread(target=read_and_write_points)
    thread.daemon = True
    thread.start()

    thread = threading.Thread(target=read_and_write_deploy_notif)
    thread.daemon = True
    thread.start()

    thread = threading.Thread(target=get_deploy_notif)
    thread.daemon = True
    thread.start()

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as server_socket:
        server_socket.bind(('0.0.0.0', LISTEN_PORT))
        server_socket.listen()

        print(f"Listening for incoming connections on port {LISTEN_PORT}...")

        while True:
            client_socket, client_address = server_socket.accept()
            ip_queue.put([time.time(), client_address[0]])
            # print(f"Connection {connection_count}: {client_address[0]}")


if __name__ == "__main__":
    main()
